@@ -1,6 +1,5 @@
 # detach(package:vcfR, unload=T)
-#
-library(testthat)
+#library(testthat)
 library(vcfR)
 
 context("AD_frequency")
